
(function($) {

  /**
   * Global, extendable functions.
   */

  Drupal.visualSelectFile = {};

  // The label form the button that opens the Views grid.
  Drupal.visualSelectFile.openButtonLabel = Drupal.t('Choose file');

  // Create the button that opens the Views grid.
  Drupal.visualSelectFile.createOpenButton = function($div) {
    var
      $textField = $div.find('.form-autocomplete'),
      $button = $('<input>')
        .attr('type', 'button')
        .attr('style', 'margin-left: 0.25em;')
        .addClass('visual-select-file-open-link')
        .addClass('form-submit')
        .val(Drupal.visualSelectFile.openButtonLabel);

    $button.click(function(e) {
      e.preventDefault();

      Drupal.visualSelectFile.openGrid($textField, e);
    });

    return $button;
  };

  // The popup window options.
  Drupal.visualSelectFile.popupWindowOptions = 'width=1200,height=750,location=no,status=yes,menubar=no,toolbar=no,resizable=yes,scrollbars=yes';

  // The name of the popup window with the Views grid.
  Drupal.visualSelectFile.popupWindowName = function($textField) {
    return 'visualSelectFile';
  };

  // The handler that opens the Views grid.
  Drupal.visualSelectFile.openGrid = function($textField, e) {
    // Get text field.
    var
      id = $textField.attr('id'),
      url = Drupal.settings.basePath + 'admin/visual_select_file#' + id,
      name = Drupal.visualSelectFile.popupWindowName($textField),
      options = Drupal.visualSelectFile.popupWindowOptions;

    open(url, name, options);
  };

  // Position/insert the button that opens the Views grid.
  Drupal.visualSelectFile.insertOpenButton = function($div, $button) {
    $div
      // Mark this instance as processed.
      .addClass('visual-select-file-processed')
      // Insert button right after FileField Sources' "Select" button.
      .find('input[type="submit"]').after($button);
  };

  // After clicking an image in the Views grid.
  Drupal.visualSelectFile.onSelect = function($td, $textField, $submit) {
    var
      $fid = $td.find('.visual-select-file-fid'),
      fid = $.trim($fid.text()),
      $name = $td.find('.visual-select-file-name'),
      name = $.trim($name.text());

    $textField.val(name + ' [fid:' + fid + ']').removeClass('hint');
    window.close();
    $submit.trigger('mousedown');
  };



  /**
   * Define global behavior.
   */

  Drupal.behaviors.visualSelectFile = {
    attach: function(context, settings) {

      /**
       * Picker page.
       */

      // Get grid TD's.
      if (location.hash && opener) {
        $('.visual-select-file-fid', context).parent().click(function(e) {
          var $td = $(this),
              $textField = opener.jQuery(location.hash),
              $submit = $textField
                .closest('.filefield-source-reference')
                .find('input[type="submit"]');

          Drupal.visualSelectFile.onSelect($td, $textField, $submit);
        });
      }



      /**
       * Node edit form.
       */

      // Get file fields.
      var fileFields = $('div.filefield-source-reference:not(.visual-select-file-processed)', context);
      fileFields.each(function(i, div) {
        var $div = $(div);

        // Create link to open popup.
        var $button = Drupal.visualSelectFile.createOpenButton($div);

        // Add link to submit button area.
        Drupal.visualSelectFile.insertOpenButton($div, $button);
      });
    }
  };

})(jQuery);
